/*****************************************************************************
 * This file is part of gfxprim library.                                     *
 *                                                                           *
 * Gfxprim is free software; you can redistribute it and/or                  *
 * modify it under the terms of the GNU Lesser General Public                *
 * License as published by the Free Software Foundation; either              *
 * version 2.1 of the License, or (at your option) any later version.        *
 *                                                                           *
 * Gfxprim is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with gfxprim; if not, write to the Free Software            *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,                        *
 * Boston, MA  02110-1301  USA                                               *
 *                                                                           *
 * Copyright (C) 2012      Cyril Hrubis <metan@ucw.cz>                       *
 *                                                                           *
 *****************************************************************************/

/*
 * GP_GammaPixel.gen.h
 *
 * GENERATED on Mon Dec 23 05:05:54 2013 from GP_GammaPixel.gen.h.t
 * DO NOT MODIFY THIS FILE DIRECTLY!
 *
 * Gamma correction for pixels.
 */
#ifndef GP_GAMMAPIXEL_GEN_H
#define GP_GAMMAPIXEL_GEN_H
#include "GP_Context.h"
#include "GP_GammaCorrection.h"


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_xRGB8888_R(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_xRGB8888_R(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_xRGB8888_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_xRGB8888_R(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_xRGB8888_G(val, gamma) ({ \
	gamma->tables[1]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_xRGB8888_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_xRGB8888_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_xRGB8888_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_xRGB8888_B(val, gamma) ({ \
	gamma->tables[2]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_xRGB8888_B(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_xRGB8888_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_xRGB8888_B(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGBA8888_R(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGBA8888_R(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGBA8888_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGBA8888_R(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGBA8888_G(val, gamma) ({ \
	gamma->tables[1]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGBA8888_G(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGBA8888_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGBA8888_G(GP_Gamma *gamma)
{
	return gamma->tables[5];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGBA8888_B(val, gamma) ({ \
	gamma->tables[2]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGBA8888_B(val, gamma) ({ \
	gamma->tables[6]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGBA8888_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGBA8888_B(GP_Gamma *gamma)
{
	return gamma->tables[6];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGBA8888_A(val, gamma) ({ \
	gamma->tables[3]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGBA8888_A(val, gamma) ({ \
	gamma->tables[7]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGBA8888_A(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGBA8888_A(GP_Gamma *gamma)
{
	return gamma->tables[7];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB888_R(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB888_R(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB888_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB888_R(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB888_G(val, gamma) ({ \
	gamma->tables[1]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB888_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB888_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB888_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB888_B(val, gamma) ({ \
	gamma->tables[2]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB888_B(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB888_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB888_B(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_BGR888_B(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_BGR888_B(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_BGR888_B(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_BGR888_B(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_BGR888_G(val, gamma) ({ \
	gamma->tables[1]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_BGR888_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_BGR888_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_BGR888_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_BGR888_R(val, gamma) ({ \
	gamma->tables[2]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_BGR888_R(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_BGR888_R(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_BGR888_R(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB555_R(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB555_R(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB555_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB555_R(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB555_G(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB555_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB555_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB555_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB555_B(val, gamma) ({ \
	gamma->tables[2]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB555_B(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB555_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB555_B(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB565_R(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB565_R(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB565_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB565_R(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB565_G(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB565_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB565_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB565_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB565_B(val, gamma) ({ \
	gamma->tables[2]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB565_B(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB565_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB565_B(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB666_R(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB666_R(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB666_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB666_R(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB666_G(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB666_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB666_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB666_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB666_B(val, gamma) ({ \
	gamma->tables[2]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB666_B(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB666_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB666_B(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB332_R(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB332_R(val, gamma) ({ \
	gamma->tables[3]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB332_R(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB332_R(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB332_G(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB332_G(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB332_G(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB332_G(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_RGB332_B(val, gamma) ({ \
	gamma->tables[2]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_RGB332_B(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_RGB332_B(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_RGB332_B(GP_Gamma *gamma)
{
	return gamma->tables[5];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_CMYK8888_K(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_CMYK8888_K(val, gamma) ({ \
	gamma->tables[4]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_CMYK8888_K(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_CMYK8888_K(GP_Gamma *gamma)
{
	return gamma->tables[4];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_CMYK8888_Y(val, gamma) ({ \
	gamma->tables[1]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_CMYK8888_Y(val, gamma) ({ \
	gamma->tables[5]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_CMYK8888_Y(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

static inline GP_GammaTable *GP_GammaInverseTable_CMYK8888_Y(GP_Gamma *gamma)
{
	return gamma->tables[5];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_CMYK8888_M(val, gamma) ({ \
	gamma->tables[2]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_CMYK8888_M(val, gamma) ({ \
	gamma->tables[6]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_CMYK8888_M(GP_Gamma *gamma)
{
	return gamma->tables[2];
}

static inline GP_GammaTable *GP_GammaInverseTable_CMYK8888_M(GP_Gamma *gamma)
{
	return gamma->tables[6];
}

/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_CMYK8888_C(val, gamma) ({ \
	gamma->tables[3]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_CMYK8888_C(val, gamma) ({ \
	gamma->tables[7]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_CMYK8888_C(GP_Gamma *gamma)
{
	return gamma->tables[3];
}

static inline GP_GammaTable *GP_GammaInverseTable_CMYK8888_C(GP_Gamma *gamma)
{
	return gamma->tables[7];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_G1_V(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_G1_V(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_G1_V(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_G1_V(GP_Gamma *gamma)
{
	return gamma->tables[1];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_G2_V(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_G2_V(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_G2_V(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_G2_V(GP_Gamma *gamma)
{
	return gamma->tables[1];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_G4_V(val, gamma) ({ \
	gamma->tables[0]->u8[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_G4_V(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_G4_V(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_G4_V(GP_Gamma *gamma)
{
	return gamma->tables[1];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_G8_V(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_G8_V(val, gamma) ({ \
	gamma->tables[1]->u8[val]; \
})

static inline GP_GammaTable *GP_GammaTable_G8_V(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_G8_V(GP_Gamma *gamma)
{
	return gamma->tables[1];
}


/*
 * Converts gamma encoded pixel value to linear value.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Gamma2Lin_G16_V(val, gamma) ({ \
	gamma->tables[0]->u16[val]; \
})

/*
 * Converts linear encoded pixel into gamma encoded pixel.
 *
 * Parameters are, converted value and GP_Gamma structure.
 */
#define GP_Lin2Gamma_G16_V(val, gamma) ({ \
	gamma->tables[1]->u16[val]; \
})

static inline GP_GammaTable *GP_GammaTable_G16_V(GP_Gamma *gamma)
{
	return gamma->tables[0];
}

static inline GP_GammaTable *GP_GammaInverseTable_G16_V(GP_Gamma *gamma)
{
	return gamma->tables[1];
}

#define GP_Gamma2Lin(val, chan_bits, gamma_table) ({ \
#if chan_bits > 6
	gamma_table->u16[val] \
#else
	gamma_table->u8[val] \
#endif
})

#define GP_Lin2Gamma(val, chan_bits, gamma_table) ({ \
#if chan_bits > 8
	gamma_table->table16[val] \
#else
	gamma_table->table8[val] \
#endif
})

#endif  /* GP_GAMMAPIXEL_GEN_H */
