/*****************************************************************************
 * This file is part of gfxprim library.                                     *
 *                                                                           *
 * Gfxprim is free software; you can redistribute it and/or                  *
 * modify it under the terms of the GNU Lesser General Public                *
 * License as published by the Free Software Foundation; either              *
 * version 2.1 of the License, or (at your option) any later version.        *
 *                                                                           *
 * Gfxprim is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with gfxprim; if not, write to the Free Software            *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,                        *
 * Boston, MA  02110-1301  USA                                               *
 *                                                                           *
 * Copyright (C) 2011      Cyril Hrubis <metan@ucw.cz>                       *
 * Copyright (C) 2011      Tomas Gavenciak <gavento@ucw.cz>                  *
 *                                                                           *
 *****************************************************************************/

/*
 * GP_GetPutPixel.gen.h
 *
 * GENERATED on Mon Dec 23 05:05:53 2013 from GP_GetPutPixel.gen.h.t
 * DO NOT MODIFY THIS FILE DIRECTLY!
 *
 * Access pixel bytes, Get and PutPixel
 * Do not include directly, use GP_Pixel.h
 */
#ifndef GP_GETPUTPIXEL_GEN_H
#define GP_GETPUTPIXEL_GEN_H
 /*

   Note about byte aligment
   ~~~~~~~~~~~~~~~~~~~~~~~~

   Unaligned access happens when instruction that works with multiple byte
   value gets an address that is not divideable by the size of the value. Eg.
   if 32 bit integer instruction gets an address that is not a multiple of 4.
   On intel cpus this type of access works and is supported however the C
   standard defines this as undefined behavior. This fails to work ARM and most
   of the non intel cpus. So some more trickery must be done in order to write
   unaligned multibyte values. First of all we must compute offset and number
   of bytes to be accessed (which is cruicial for speed as we are going to read
   the pixel value byte by byte).

   The offsets (starting with the first one eg. pixel_size mod 8) forms subgroup
   in the mod 8 cyclic group. The maximal count of bits, from the start of the
   byte, then will be max from this subgroup + pixel_size. If this number is
   less or equal to 8 * N, we could write such pixel by writing N bytes.

   For example the offsets of 16 BPP forms subgroup only with {0} so we only
   need 2 bytes to write it. As a matter of fact the 16 and 32 BPP are special
   cases that are always aligned together with the 8 BPP (which is aligned
   trivially). These three are coded as special cases which yields to faster
   operations in case of 16 and 32 BPP. The 24 BPP is not aligned as there are
   no instruction to operate 3 byte long numbers.

   For second example take offsets of 20 BPP that forms subgroup {4, 0}
   so the max + pixel_size = 24 and indeed we fit into 3 bytes.

   If pixel_size is coprime to 8, the offsets generates whole group and so the
   max + pixel_size = 7 + pixel_size. The 17 BPP fits into 24 bits and so 3
   bytes are needed. The 19 BPP fits into 26 bits and because of that 4 bytes
   are needed.

   Once we figure maximal number of bytes and the offset all that is to be done
   is to fetch first and last byte to combine it together with given pixel value
   and write the result back to the bitmap.

 */

#include "GP_GetSetBits.h"
#include "GP_Context.h"

/*
 * macro to get address of pixel in a 1BPP_LE context
 */
#define GP_PIXEL_ADDR_1BPP_LE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (1 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 1BPP_LE context
 */
#define GP_PIXEL_ADDR_OFFSET_1BPP_LE(x) \
	(((x) % 8) * 1)
/*
 * GP_GetPixel for 1BPP_LE
 */
static inline GP_Pixel GP_GetPixel_Raw_1BPP_LE(const GP_Context *c, int x, int y)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_1BPP_LE(x), 1,
		*(GP_PIXEL_ADDR_1BPP_LE(c, x, y)));
}

/*
 * GP_PutPixel for 1BPP_LE
 */
static inline void GP_PutPixel_Raw_1BPP_LE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_1BPP_LE(x), 1,
	                     GP_PIXEL_ADDR_1BPP_LE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_1BPP_LE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_1BPP_LE(c, x, y, p);
}

/*
 * macro to get address of pixel in a 1BPP_BE context
 */
#define GP_PIXEL_ADDR_1BPP_BE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (1 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 1BPP_BE context
 */
#define GP_PIXEL_ADDR_OFFSET_1BPP_BE(x) \
	(7 - ((x) % 8) * 1)
/*
 * GP_GetPixel for 1BPP_BE
 */
static inline GP_Pixel GP_GetPixel_Raw_1BPP_BE(const GP_Context *c, int x, int y)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_1BPP_BE(x), 1,
		*(GP_PIXEL_ADDR_1BPP_BE(c, x, y)));
}

/*
 * GP_PutPixel for 1BPP_BE
 */
static inline void GP_PutPixel_Raw_1BPP_BE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_1BPP_BE(x), 1,
	                     GP_PIXEL_ADDR_1BPP_BE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_1BPP_BE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_1BPP_BE(c, x, y, p);
}

/*
 * macro to get address of pixel in a 2BPP_LE context
 */
#define GP_PIXEL_ADDR_2BPP_LE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (2 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 2BPP_LE context
 */
#define GP_PIXEL_ADDR_OFFSET_2BPP_LE(x) \
	(((x) % 4) * 2)
/*
 * GP_GetPixel for 2BPP_LE
 */
static inline GP_Pixel GP_GetPixel_Raw_2BPP_LE(const GP_Context *c, int x, int y)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_2BPP_LE(x), 2,
		*(GP_PIXEL_ADDR_2BPP_LE(c, x, y)));
}

/*
 * GP_PutPixel for 2BPP_LE
 */
static inline void GP_PutPixel_Raw_2BPP_LE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_2BPP_LE(x), 2,
	                     GP_PIXEL_ADDR_2BPP_LE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_2BPP_LE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_2BPP_LE(c, x, y, p);
}

/*
 * macro to get address of pixel in a 2BPP_BE context
 */
#define GP_PIXEL_ADDR_2BPP_BE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (2 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 2BPP_BE context
 */
#define GP_PIXEL_ADDR_OFFSET_2BPP_BE(x) \
	(6 - ((x) % 4) * 2)
/*
 * GP_GetPixel for 2BPP_BE
 */
static inline GP_Pixel GP_GetPixel_Raw_2BPP_BE(const GP_Context *c, int x, int y)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_2BPP_BE(x), 2,
		*(GP_PIXEL_ADDR_2BPP_BE(c, x, y)));
}

/*
 * GP_PutPixel for 2BPP_BE
 */
static inline void GP_PutPixel_Raw_2BPP_BE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_2BPP_BE(x), 2,
	                     GP_PIXEL_ADDR_2BPP_BE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_2BPP_BE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_2BPP_BE(c, x, y, p);
}

/*
 * macro to get address of pixel in a 4BPP_LE context
 */
#define GP_PIXEL_ADDR_4BPP_LE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (4 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 4BPP_LE context
 */
#define GP_PIXEL_ADDR_OFFSET_4BPP_LE(x) \
	(((x) % 2) * 4)
/*
 * GP_GetPixel for 4BPP_LE
 */
static inline GP_Pixel GP_GetPixel_Raw_4BPP_LE(const GP_Context *c, int x, int y)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_4BPP_LE(x), 4,
		*(GP_PIXEL_ADDR_4BPP_LE(c, x, y)));
}

/*
 * GP_PutPixel for 4BPP_LE
 */
static inline void GP_PutPixel_Raw_4BPP_LE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_4BPP_LE(x), 4,
	                     GP_PIXEL_ADDR_4BPP_LE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_4BPP_LE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_4BPP_LE(c, x, y, p);
}

/*
 * macro to get address of pixel in a 4BPP_BE context
 */
#define GP_PIXEL_ADDR_4BPP_BE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (4 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 4BPP_BE context
 */
#define GP_PIXEL_ADDR_OFFSET_4BPP_BE(x) \
	(4 - ((x) % 2) * 4)
/*
 * GP_GetPixel for 4BPP_BE
 */
static inline GP_Pixel GP_GetPixel_Raw_4BPP_BE(const GP_Context *c, int x, int y)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_4BPP_BE(x), 4,
		*(GP_PIXEL_ADDR_4BPP_BE(c, x, y)));
}

/*
 * GP_PutPixel for 4BPP_BE
 */
static inline void GP_PutPixel_Raw_4BPP_BE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * Whole pixel is stored only and only in one byte
	 *
	 * The full list = {1, 2, 4, 8}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_OFFSET_4BPP_BE(x), 4,
	                     GP_PIXEL_ADDR_4BPP_BE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_4BPP_BE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_4BPP_BE(c, x, y, p);
}

/*
 * macro to get address of pixel in a 8BPP context
 */
#define GP_PIXEL_ADDR_8BPP(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (8 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 8BPP context
 */
#define GP_PIXEL_ADDR_OFFSET_8BPP(x) \
	(0)
/*
 * GP_GetPixel for 8BPP
 */
static inline GP_Pixel GP_GetPixel_Raw_8BPP(const GP_Context *c, int x, int y)
{
	/*
	 * 8 BPP is byte aligned
	 */
	return *((uint8_t*)GP_PIXEL_ADDR_8BPP(c, x, y));
}

/*
 * GP_PutPixel for 8BPP
 */
static inline void GP_PutPixel_Raw_8BPP(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * 8 BPP is byte aligned
	 */
	*((uint8_t*)GP_PIXEL_ADDR_8BPP(c, x, y)) = p;
}

static inline void GP_PutPixel_Raw_Clipped_8BPP(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_8BPP(c, x, y, p);
}

/*
 * macro to get address of pixel in a 16BPP context
 */
#define GP_PIXEL_ADDR_16BPP(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (16 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 16BPP context
 */
#define GP_PIXEL_ADDR_OFFSET_16BPP(x) \
	(0)
/*
 * GP_GetPixel for 16BPP
 */
static inline GP_Pixel GP_GetPixel_Raw_16BPP(const GP_Context *c, int x, int y)
{
	/*
	 * 16 BPP is expected to have aligned pixels
	 */
	return *((uint16_t*)GP_PIXEL_ADDR_16BPP(c, x, y));
}

/*
 * GP_PutPixel for 16BPP
 */
static inline void GP_PutPixel_Raw_16BPP(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * 16 BPP is expected to have aligned pixels
	 */
	*((uint16_t*)GP_PIXEL_ADDR_16BPP(c, x, y)) = p;
}

static inline void GP_PutPixel_Raw_Clipped_16BPP(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_16BPP(c, x, y, p);
}

/*
 * macro to get address of pixel in a 24BPP context
 */
#define GP_PIXEL_ADDR_24BPP(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (24 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 24BPP context
 */
#define GP_PIXEL_ADDR_OFFSET_24BPP(x) \
	(0)
/*
 * GP_GetPixel for 24BPP
 */
static inline GP_Pixel GP_GetPixel_Raw_24BPP(const GP_Context *c, int x, int y)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	return GP_GET_BITS3_ALIGNED(GP_PIXEL_ADDR_OFFSET_24BPP(x), 24,
		*(GP_PIXEL_ADDR_24BPP(c, x, y)));
}

/*
 * GP_PutPixel for 24BPP
 */
static inline void GP_PutPixel_Raw_24BPP(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	GP_SET_BITS3_ALIGNED(GP_PIXEL_ADDR_OFFSET_24BPP(x), 24,
	                     GP_PIXEL_ADDR_24BPP(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_24BPP(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_24BPP(c, x, y, p);
}

/*
 * macro to get address of pixel in a 32BPP context
 */
#define GP_PIXEL_ADDR_32BPP(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (32 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 32BPP context
 */
#define GP_PIXEL_ADDR_OFFSET_32BPP(x) \
	(0)
/*
 * GP_GetPixel for 32BPP
 */
static inline GP_Pixel GP_GetPixel_Raw_32BPP(const GP_Context *c, int x, int y)
{
	/*
	 * 32 BPP is expected to have aligned pixels
	 */
	return *((uint32_t*)GP_PIXEL_ADDR_32BPP(c, x, y));
}

/*
 * GP_PutPixel for 32BPP
 */
static inline void GP_PutPixel_Raw_32BPP(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * 32 BPP is expected to have aligned pixels
	 */
	*((uint32_t*)GP_PIXEL_ADDR_32BPP(c, x, y)) = p;
}

static inline void GP_PutPixel_Raw_Clipped_32BPP(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_32BPP(c, x, y, p);
}

/*
 * macro to get address of pixel in a 18BPP_LE context
 */
#define GP_PIXEL_ADDR_18BPP_LE(context, x, y) \
	((GP_Pixel*)(((void*)((context)->pixels)) + (context)->bytes_per_row * (y) + (18 * (x)) / 8))

/*
 * macro to get bit-offset of pixel in 18BPP_LE context
 */
#define GP_PIXEL_ADDR_OFFSET_18BPP_LE(x) \
	((18 * (x)) % 8)
/*
 * GP_GetPixel for 18BPP_LE
 */
static inline GP_Pixel GP_GetPixel_Raw_18BPP_LE(const GP_Context *c, int x, int y)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	return GP_GET_BITS3_ALIGNED(GP_PIXEL_ADDR_OFFSET_18BPP_LE(x), 18,
		*(GP_PIXEL_ADDR_18BPP_LE(c, x, y)));
}

/*
 * GP_PutPixel for 18BPP_LE
 */
static inline void GP_PutPixel_Raw_18BPP_LE(GP_Context *c, int x, int y, GP_Pixel p)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	GP_SET_BITS3_ALIGNED(GP_PIXEL_ADDR_OFFSET_18BPP_LE(x), 18,
	                     GP_PIXEL_ADDR_18BPP_LE(c, x, y), p);
}

static inline void GP_PutPixel_Raw_Clipped_18BPP_LE(GP_Context *c, GP_Coord x, GP_Coord y, GP_Pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	GP_PutPixel_Raw_18BPP_LE(c, x, y, p);
}


#endif  /* GP_GETPUTPIXEL_GEN_H */
