/*****************************************************************************
 * This file is part of gfxprim library.                                     *
 *                                                                           *
 * Gfxprim is free software; you can redistribute it and/or                  *
 * modify it under the terms of the GNU Lesser General Public                *
 * License as published by the Free Software Foundation; either              *
 * version 2.1 of the License, or (at your option) any later version.        *
 *                                                                           *
 * Gfxprim is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with gfxprim; if not, write to the Free Software            *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,                        *
 * Boston, MA  02110-1301  USA                                               *
 *                                                                           *
 * Copyright (C) 2011-2012 Cyril Hrubis <metan@ucw.cz>                       *
 *                                                                           *
 *****************************************************************************/

/*
 * GP_MixPixels.gen.h
 *
 * GENERATED on Mon Dec 23 05:05:54 2013 from GP_MixPixels.gen.h.t
 * DO NOT MODIFY THIS FILE DIRECTLY!
 *
 * Macros to mix two pixels accordingly to percentage.
 */
#ifndef GP_MIXPIXELS_GEN_H
#define GP_MIXPIXELS_GEN_H
#include "core/GP_Context.h"
#include "core/GP_Pixel.h"
#include "core/GP_GetPutPixel.h"
#include "core/GP_GammaCorrection.h"

/*
 * Mixes two xRGB8888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_xRGB8888(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_xRGB8888(pix1) * (perc); \
	R += GP_Pixel_GET_R_xRGB8888(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_xRGB8888(pix1) * (perc); \
	G += GP_Pixel_GET_G_xRGB8888(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_xRGB8888(pix1) * (perc); \
	B += GP_Pixel_GET_B_xRGB8888(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
\
	GP_Pixel_CREATE_xRGB8888(R, G, B); \
})

/*
 * Mixes two xRGB8888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_xRGB8888(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma8ToLinear10(GP_Pixel_GET_R_xRGB8888(pix1)) * (perc); \
	R += GP_Gamma8ToLinear10(GP_Pixel_GET_R_xRGB8888(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma8(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma8ToLinear10(GP_Pixel_GET_G_xRGB8888(pix1)) * (perc); \
	G += GP_Gamma8ToLinear10(GP_Pixel_GET_G_xRGB8888(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma8(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma8ToLinear10(GP_Pixel_GET_B_xRGB8888(pix1)) * (perc); \
	B += GP_Gamma8ToLinear10(GP_Pixel_GET_B_xRGB8888(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma8(B); \
\
\
	GP_Pixel_CREATE_xRGB8888(R, G, B); \
})

#define GP_MIX_PIXELS_xRGB8888(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_xRGB8888(pix1, pix2, perc)

/*
 * Mixes two RGBA8888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_RGBA8888(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_RGBA8888(pix1) * (perc); \
	R += GP_Pixel_GET_R_RGBA8888(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_RGBA8888(pix1) * (perc); \
	G += GP_Pixel_GET_G_RGBA8888(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_RGBA8888(pix1) * (perc); \
	B += GP_Pixel_GET_B_RGBA8888(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
	GP_Pixel A; \
\
	A  = GP_Pixel_GET_A_RGBA8888(pix1) * (perc); \
	A += GP_Pixel_GET_A_RGBA8888(pix2) * (255 - (perc)); \
	A = (A + 128) / 255; \
\
\
	GP_Pixel_CREATE_RGBA8888(R, G, B, A); \
})

/*
 * Mixes two RGBA8888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_RGBA8888(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma8ToLinear10(GP_Pixel_GET_R_RGBA8888(pix1)) * (perc); \
	R += GP_Gamma8ToLinear10(GP_Pixel_GET_R_RGBA8888(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma8(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma8ToLinear10(GP_Pixel_GET_G_RGBA8888(pix1)) * (perc); \
	G += GP_Gamma8ToLinear10(GP_Pixel_GET_G_RGBA8888(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma8(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma8ToLinear10(GP_Pixel_GET_B_RGBA8888(pix1)) * (perc); \
	B += GP_Gamma8ToLinear10(GP_Pixel_GET_B_RGBA8888(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma8(B); \
\
	GP_Pixel A; \
\
	A  = GP_Gamma8ToLinear10(GP_Pixel_GET_A_RGBA8888(pix1)) * (perc); \
	A += GP_Gamma8ToLinear10(GP_Pixel_GET_A_RGBA8888(pix2)) * (255 - (perc)); \
	A = (A + 128) / 255; \
	A = GP_Linear10ToGamma8(A); \
\
\
	GP_Pixel_CREATE_RGBA8888(R, G, B, A); \
})

#define GP_MIX_PIXELS_RGBA8888(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_RGBA8888(pix1, pix2, perc)

/*
 * Mixes two RGB888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_RGB888(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_RGB888(pix1) * (perc); \
	R += GP_Pixel_GET_R_RGB888(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_RGB888(pix1) * (perc); \
	G += GP_Pixel_GET_G_RGB888(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_RGB888(pix1) * (perc); \
	B += GP_Pixel_GET_B_RGB888(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
\
	GP_Pixel_CREATE_RGB888(R, G, B); \
})

/*
 * Mixes two RGB888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_RGB888(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma8ToLinear10(GP_Pixel_GET_R_RGB888(pix1)) * (perc); \
	R += GP_Gamma8ToLinear10(GP_Pixel_GET_R_RGB888(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma8(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma8ToLinear10(GP_Pixel_GET_G_RGB888(pix1)) * (perc); \
	G += GP_Gamma8ToLinear10(GP_Pixel_GET_G_RGB888(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma8(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma8ToLinear10(GP_Pixel_GET_B_RGB888(pix1)) * (perc); \
	B += GP_Gamma8ToLinear10(GP_Pixel_GET_B_RGB888(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma8(B); \
\
\
	GP_Pixel_CREATE_RGB888(R, G, B); \
})

#define GP_MIX_PIXELS_RGB888(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_RGB888(pix1, pix2, perc)

/*
 * Mixes two BGR888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_BGR888(pix1, pix2, perc) ({ \
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_BGR888(pix1) * (perc); \
	B += GP_Pixel_GET_B_BGR888(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_BGR888(pix1) * (perc); \
	G += GP_Pixel_GET_G_BGR888(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_BGR888(pix1) * (perc); \
	R += GP_Pixel_GET_R_BGR888(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
\
	GP_Pixel_CREATE_BGR888(B, G, R); \
})

/*
 * Mixes two BGR888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_BGR888(pix1, pix2, perc) ({ \
	GP_Pixel B; \
\
	B  = GP_Gamma8ToLinear10(GP_Pixel_GET_B_BGR888(pix1)) * (perc); \
	B += GP_Gamma8ToLinear10(GP_Pixel_GET_B_BGR888(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma8(B); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma8ToLinear10(GP_Pixel_GET_G_BGR888(pix1)) * (perc); \
	G += GP_Gamma8ToLinear10(GP_Pixel_GET_G_BGR888(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma8(G); \
\
	GP_Pixel R; \
\
	R  = GP_Gamma8ToLinear10(GP_Pixel_GET_R_BGR888(pix1)) * (perc); \
	R += GP_Gamma8ToLinear10(GP_Pixel_GET_R_BGR888(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma8(R); \
\
\
	GP_Pixel_CREATE_BGR888(B, G, R); \
})

#define GP_MIX_PIXELS_BGR888(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_BGR888(pix1, pix2, perc)

/*
 * Mixes two RGB555 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_RGB555(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_RGB555(pix1) * (perc); \
	R += GP_Pixel_GET_R_RGB555(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_RGB555(pix1) * (perc); \
	G += GP_Pixel_GET_G_RGB555(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_RGB555(pix1) * (perc); \
	B += GP_Pixel_GET_B_RGB555(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
\
	GP_Pixel_CREATE_RGB555(R, G, B); \
})

/*
 * Mixes two RGB555 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_RGB555(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma5ToLinear10(GP_Pixel_GET_R_RGB555(pix1)) * (perc); \
	R += GP_Gamma5ToLinear10(GP_Pixel_GET_R_RGB555(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma5(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma5ToLinear10(GP_Pixel_GET_G_RGB555(pix1)) * (perc); \
	G += GP_Gamma5ToLinear10(GP_Pixel_GET_G_RGB555(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma5(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma5ToLinear10(GP_Pixel_GET_B_RGB555(pix1)) * (perc); \
	B += GP_Gamma5ToLinear10(GP_Pixel_GET_B_RGB555(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma5(B); \
\
\
	GP_Pixel_CREATE_RGB555(R, G, B); \
})

#define GP_MIX_PIXELS_RGB555(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_RGB555(pix1, pix2, perc)

/*
 * Mixes two RGB565 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_RGB565(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_RGB565(pix1) * (perc); \
	R += GP_Pixel_GET_R_RGB565(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_RGB565(pix1) * (perc); \
	G += GP_Pixel_GET_G_RGB565(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_RGB565(pix1) * (perc); \
	B += GP_Pixel_GET_B_RGB565(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
\
	GP_Pixel_CREATE_RGB565(R, G, B); \
})

/*
 * Mixes two RGB565 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_RGB565(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma5ToLinear10(GP_Pixel_GET_R_RGB565(pix1)) * (perc); \
	R += GP_Gamma5ToLinear10(GP_Pixel_GET_R_RGB565(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma5(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma6ToLinear10(GP_Pixel_GET_G_RGB565(pix1)) * (perc); \
	G += GP_Gamma6ToLinear10(GP_Pixel_GET_G_RGB565(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma6(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma5ToLinear10(GP_Pixel_GET_B_RGB565(pix1)) * (perc); \
	B += GP_Gamma5ToLinear10(GP_Pixel_GET_B_RGB565(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma5(B); \
\
\
	GP_Pixel_CREATE_RGB565(R, G, B); \
})

#define GP_MIX_PIXELS_RGB565(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_RGB565(pix1, pix2, perc)

/*
 * Mixes two RGB666 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_RGB666(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_RGB666(pix1) * (perc); \
	R += GP_Pixel_GET_R_RGB666(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_RGB666(pix1) * (perc); \
	G += GP_Pixel_GET_G_RGB666(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_RGB666(pix1) * (perc); \
	B += GP_Pixel_GET_B_RGB666(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
\
	GP_Pixel_CREATE_RGB666(R, G, B); \
})

/*
 * Mixes two RGB666 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_RGB666(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma6ToLinear10(GP_Pixel_GET_R_RGB666(pix1)) * (perc); \
	R += GP_Gamma6ToLinear10(GP_Pixel_GET_R_RGB666(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma6(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma6ToLinear10(GP_Pixel_GET_G_RGB666(pix1)) * (perc); \
	G += GP_Gamma6ToLinear10(GP_Pixel_GET_G_RGB666(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma6(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma6ToLinear10(GP_Pixel_GET_B_RGB666(pix1)) * (perc); \
	B += GP_Gamma6ToLinear10(GP_Pixel_GET_B_RGB666(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma6(B); \
\
\
	GP_Pixel_CREATE_RGB666(R, G, B); \
})

#define GP_MIX_PIXELS_RGB666(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_RGB666(pix1, pix2, perc)

/*
 * Mixes two RGB332 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_RGB332(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Pixel_GET_R_RGB332(pix1) * (perc); \
	R += GP_Pixel_GET_R_RGB332(pix2) * (255 - (perc)); \
	R = (R + 128) / 255; \
\
	GP_Pixel G; \
\
	G  = GP_Pixel_GET_G_RGB332(pix1) * (perc); \
	G += GP_Pixel_GET_G_RGB332(pix2) * (255 - (perc)); \
	G = (G + 128) / 255; \
\
	GP_Pixel B; \
\
	B  = GP_Pixel_GET_B_RGB332(pix1) * (perc); \
	B += GP_Pixel_GET_B_RGB332(pix2) * (255 - (perc)); \
	B = (B + 128) / 255; \
\
\
	GP_Pixel_CREATE_RGB332(R, G, B); \
})

/*
 * Mixes two RGB332 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_RGB332(pix1, pix2, perc) ({ \
	GP_Pixel R; \
\
	R  = GP_Gamma3ToLinear10(GP_Pixel_GET_R_RGB332(pix1)) * (perc); \
	R += GP_Gamma3ToLinear10(GP_Pixel_GET_R_RGB332(pix2)) * (255 - (perc)); \
	R = (R + 128) / 255; \
	R = GP_Linear10ToGamma3(R); \
\
	GP_Pixel G; \
\
	G  = GP_Gamma3ToLinear10(GP_Pixel_GET_G_RGB332(pix1)) * (perc); \
	G += GP_Gamma3ToLinear10(GP_Pixel_GET_G_RGB332(pix2)) * (255 - (perc)); \
	G = (G + 128) / 255; \
	G = GP_Linear10ToGamma3(G); \
\
	GP_Pixel B; \
\
	B  = GP_Gamma2ToLinear10(GP_Pixel_GET_B_RGB332(pix1)) * (perc); \
	B += GP_Gamma2ToLinear10(GP_Pixel_GET_B_RGB332(pix2)) * (255 - (perc)); \
	B = (B + 128) / 255; \
	B = GP_Linear10ToGamma2(B); \
\
\
	GP_Pixel_CREATE_RGB332(R, G, B); \
})

#define GP_MIX_PIXELS_RGB332(pix1, pix2, perc) \
	GP_MIX_PIXELS_GAMMA_RGB332(pix1, pix2, perc)

/*
 * Mixes two CMYK8888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_CMYK8888(pix1, pix2, perc) ({ \
	GP_Pixel K; \
\
	K  = GP_Pixel_GET_K_CMYK8888(pix1) * (perc); \
	K += GP_Pixel_GET_K_CMYK8888(pix2) * (255 - (perc)); \
	K = (K + 128) / 255; \
\
	GP_Pixel Y; \
\
	Y  = GP_Pixel_GET_Y_CMYK8888(pix1) * (perc); \
	Y += GP_Pixel_GET_Y_CMYK8888(pix2) * (255 - (perc)); \
	Y = (Y + 128) / 255; \
\
	GP_Pixel M; \
\
	M  = GP_Pixel_GET_M_CMYK8888(pix1) * (perc); \
	M += GP_Pixel_GET_M_CMYK8888(pix2) * (255 - (perc)); \
	M = (M + 128) / 255; \
\
	GP_Pixel C; \
\
	C  = GP_Pixel_GET_C_CMYK8888(pix1) * (perc); \
	C += GP_Pixel_GET_C_CMYK8888(pix2) * (255 - (perc)); \
	C = (C + 128) / 255; \
\
\
	GP_Pixel_CREATE_CMYK8888(K, Y, M, C); \
})

/*
 * Mixes two CMYK8888 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_CMYK8888(pix1, pix2, perc) ({ \
	GP_Pixel K; \
\
	K  = GP_Gamma8ToLinear10(GP_Pixel_GET_K_CMYK8888(pix1)) * (perc); \
	K += GP_Gamma8ToLinear10(GP_Pixel_GET_K_CMYK8888(pix2)) * (255 - (perc)); \
	K = (K + 128) / 255; \
	K = GP_Linear10ToGamma8(K); \
\
	GP_Pixel Y; \
\
	Y  = GP_Gamma8ToLinear10(GP_Pixel_GET_Y_CMYK8888(pix1)) * (perc); \
	Y += GP_Gamma8ToLinear10(GP_Pixel_GET_Y_CMYK8888(pix2)) * (255 - (perc)); \
	Y = (Y + 128) / 255; \
	Y = GP_Linear10ToGamma8(Y); \
\
	GP_Pixel M; \
\
	M  = GP_Gamma8ToLinear10(GP_Pixel_GET_M_CMYK8888(pix1)) * (perc); \
	M += GP_Gamma8ToLinear10(GP_Pixel_GET_M_CMYK8888(pix2)) * (255 - (perc)); \
	M = (M + 128) / 255; \
	M = GP_Linear10ToGamma8(M); \
\
	GP_Pixel C; \
\
	C  = GP_Gamma8ToLinear10(GP_Pixel_GET_C_CMYK8888(pix1)) * (perc); \
	C += GP_Gamma8ToLinear10(GP_Pixel_GET_C_CMYK8888(pix2)) * (255 - (perc)); \
	C = (C + 128) / 255; \
	C = GP_Linear10ToGamma8(C); \
\
\
	GP_Pixel_CREATE_CMYK8888(K, Y, M, C); \
})

#define GP_MIX_PIXELS_CMYK8888(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_CMYK8888(pix1, pix2, perc)

/*
 * Mixes two P2 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_P2(pix1, pix2, perc) ({ \
	GP_Pixel P; \
\
	P  = GP_Pixel_GET_P_P2(pix1) * (perc); \
	P += GP_Pixel_GET_P_P2(pix2) * (255 - (perc)); \
	P = (P + 128) / 255; \
\
\
	GP_Pixel_CREATE_P2(P); \
})

/*
 * Mixes two P2 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_P2(pix1, pix2, perc) ({ \
	GP_Pixel P; \
\
	P  = GP_Gamma2ToLinear10(GP_Pixel_GET_P_P2(pix1)) * (perc); \
	P += GP_Gamma2ToLinear10(GP_Pixel_GET_P_P2(pix2)) * (255 - (perc)); \
	P = (P + 128) / 255; \
	P = GP_Linear10ToGamma2(P); \
\
\
	GP_Pixel_CREATE_P2(P); \
})

#define GP_MIX_PIXELS_P2(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_P2(pix1, pix2, perc)

/*
 * Mixes two P4 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_P4(pix1, pix2, perc) ({ \
	GP_Pixel P; \
\
	P  = GP_Pixel_GET_P_P4(pix1) * (perc); \
	P += GP_Pixel_GET_P_P4(pix2) * (255 - (perc)); \
	P = (P + 128) / 255; \
\
\
	GP_Pixel_CREATE_P4(P); \
})

/*
 * Mixes two P4 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_P4(pix1, pix2, perc) ({ \
	GP_Pixel P; \
\
	P  = GP_Gamma4ToLinear10(GP_Pixel_GET_P_P4(pix1)) * (perc); \
	P += GP_Gamma4ToLinear10(GP_Pixel_GET_P_P4(pix2)) * (255 - (perc)); \
	P = (P + 128) / 255; \
	P = GP_Linear10ToGamma4(P); \
\
\
	GP_Pixel_CREATE_P4(P); \
})

#define GP_MIX_PIXELS_P4(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_P4(pix1, pix2, perc)

/*
 * Mixes two P8 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_P8(pix1, pix2, perc) ({ \
	GP_Pixel P; \
\
	P  = GP_Pixel_GET_P_P8(pix1) * (perc); \
	P += GP_Pixel_GET_P_P8(pix2) * (255 - (perc)); \
	P = (P + 128) / 255; \
\
\
	GP_Pixel_CREATE_P8(P); \
})

/*
 * Mixes two P8 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_P8(pix1, pix2, perc) ({ \
	GP_Pixel P; \
\
	P  = GP_Gamma8ToLinear10(GP_Pixel_GET_P_P8(pix1)) * (perc); \
	P += GP_Gamma8ToLinear10(GP_Pixel_GET_P_P8(pix2)) * (255 - (perc)); \
	P = (P + 128) / 255; \
	P = GP_Linear10ToGamma8(P); \
\
\
	GP_Pixel_CREATE_P8(P); \
})

#define GP_MIX_PIXELS_P8(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_P8(pix1, pix2, perc)

/*
 * Mixes two G1 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_G1(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Pixel_GET_V_G1(pix1) * (perc); \
	V += GP_Pixel_GET_V_G1(pix2) * (255 - (perc)); \
	V = (V + 128) / 255; \
\
\
	GP_Pixel_CREATE_G1(V); \
})

/*
 * Mixes two G1 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_G1(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Gamma1ToLinear10(GP_Pixel_GET_V_G1(pix1)) * (perc); \
	V += GP_Gamma1ToLinear10(GP_Pixel_GET_V_G1(pix2)) * (255 - (perc)); \
	V = (V + 128) / 255; \
	V = GP_Linear10ToGamma1(V); \
\
\
	GP_Pixel_CREATE_G1(V); \
})

#define GP_MIX_PIXELS_G1(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_G1(pix1, pix2, perc)

/*
 * Mixes two G2 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_G2(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Pixel_GET_V_G2(pix1) * (perc); \
	V += GP_Pixel_GET_V_G2(pix2) * (255 - (perc)); \
	V = (V + 128) / 255; \
\
\
	GP_Pixel_CREATE_G2(V); \
})

/*
 * Mixes two G2 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_G2(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Gamma2ToLinear10(GP_Pixel_GET_V_G2(pix1)) * (perc); \
	V += GP_Gamma2ToLinear10(GP_Pixel_GET_V_G2(pix2)) * (255 - (perc)); \
	V = (V + 128) / 255; \
	V = GP_Linear10ToGamma2(V); \
\
\
	GP_Pixel_CREATE_G2(V); \
})

#define GP_MIX_PIXELS_G2(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_G2(pix1, pix2, perc)

/*
 * Mixes two G4 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_G4(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Pixel_GET_V_G4(pix1) * (perc); \
	V += GP_Pixel_GET_V_G4(pix2) * (255 - (perc)); \
	V = (V + 128) / 255; \
\
\
	GP_Pixel_CREATE_G4(V); \
})

/*
 * Mixes two G4 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_G4(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Gamma4ToLinear10(GP_Pixel_GET_V_G4(pix1)) * (perc); \
	V += GP_Gamma4ToLinear10(GP_Pixel_GET_V_G4(pix2)) * (255 - (perc)); \
	V = (V + 128) / 255; \
	V = GP_Linear10ToGamma4(V); \
\
\
	GP_Pixel_CREATE_G4(V); \
})

#define GP_MIX_PIXELS_G4(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_G4(pix1, pix2, perc)

/*
 * Mixes two G8 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_G8(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Pixel_GET_V_G8(pix1) * (perc); \
	V += GP_Pixel_GET_V_G8(pix2) * (255 - (perc)); \
	V = (V + 128) / 255; \
\
\
	GP_Pixel_CREATE_G8(V); \
})

/*
 * Mixes two G8 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_G8(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Gamma8ToLinear10(GP_Pixel_GET_V_G8(pix1)) * (perc); \
	V += GP_Gamma8ToLinear10(GP_Pixel_GET_V_G8(pix2)) * (255 - (perc)); \
	V = (V + 128) / 255; \
	V = GP_Linear10ToGamma8(V); \
\
\
	GP_Pixel_CREATE_G8(V); \
})

#define GP_MIX_PIXELS_G8(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_G8(pix1, pix2, perc)

/*
 * Mixes two G16 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_LINEAR_G16(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Pixel_GET_V_G16(pix1) * (perc); \
	V += GP_Pixel_GET_V_G16(pix2) * (255 - (perc)); \
	V = (V + 128) / 255; \
\
\
	GP_Pixel_CREATE_G16(V); \
})

/*
 * Mixes two G16 pixels.
 *
 * The percentage is expected as 8 bit unsigned integer [0 .. 255]
 */
#define GP_MIX_PIXELS_GAMMA_G16(pix1, pix2, perc) ({ \
	GP_Pixel V; \
\
	V  = GP_Gamma16ToLinear10(GP_Pixel_GET_V_G16(pix1)) * (perc); \
	V += GP_Gamma16ToLinear10(GP_Pixel_GET_V_G16(pix2)) * (255 - (perc)); \
	V = (V + 128) / 255; \
	V = GP_Linear10ToGamma16(V); \
\
\
	GP_Pixel_CREATE_G16(V); \
})

#define GP_MIX_PIXELS_G16(pix1, pix2, perc) \
	GP_MIX_PIXELS_LINEAR_G16(pix1, pix2, perc)

static inline GP_Pixel GP_MixPixels(GP_Pixel pix1, GP_Pixel pix2,
                                    uint8_t perc, GP_PixelType pixel_type)
{
	switch (pixel_type) {
	case GP_PIXEL_xRGB8888:
		return GP_MIX_PIXELS_LINEAR_xRGB8888(pix1, pix2, perc);
	case GP_PIXEL_RGBA8888:
		return GP_MIX_PIXELS_LINEAR_RGBA8888(pix1, pix2, perc);
	case GP_PIXEL_RGB888:
		return GP_MIX_PIXELS_LINEAR_RGB888(pix1, pix2, perc);
	case GP_PIXEL_BGR888:
		return GP_MIX_PIXELS_LINEAR_BGR888(pix1, pix2, perc);
	case GP_PIXEL_RGB555:
		return GP_MIX_PIXELS_LINEAR_RGB555(pix1, pix2, perc);
	case GP_PIXEL_RGB565:
		return GP_MIX_PIXELS_LINEAR_RGB565(pix1, pix2, perc);
	case GP_PIXEL_RGB666:
		return GP_MIX_PIXELS_LINEAR_RGB666(pix1, pix2, perc);
	case GP_PIXEL_RGB332:
		return GP_MIX_PIXELS_LINEAR_RGB332(pix1, pix2, perc);
	case GP_PIXEL_CMYK8888:
		return GP_MIX_PIXELS_LINEAR_CMYK8888(pix1, pix2, perc);
	case GP_PIXEL_P2:
		return GP_MIX_PIXELS_LINEAR_P2(pix1, pix2, perc);
	case GP_PIXEL_P4:
		return GP_MIX_PIXELS_LINEAR_P4(pix1, pix2, perc);
	case GP_PIXEL_P8:
		return GP_MIX_PIXELS_LINEAR_P8(pix1, pix2, perc);
	case GP_PIXEL_G1:
		return GP_MIX_PIXELS_LINEAR_G1(pix1, pix2, perc);
	case GP_PIXEL_G2:
		return GP_MIX_PIXELS_LINEAR_G2(pix1, pix2, perc);
	case GP_PIXEL_G4:
		return GP_MIX_PIXELS_LINEAR_G4(pix1, pix2, perc);
	case GP_PIXEL_G8:
		return GP_MIX_PIXELS_LINEAR_G8(pix1, pix2, perc);
	case GP_PIXEL_G16:
		return GP_MIX_PIXELS_LINEAR_G16(pix1, pix2, perc);
	default:
		GP_ABORT("Unknown pixeltype");
	}
}


static inline void GP_MixPixel_Raw_xRGB8888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_32BPP(context, x, y);
	pix = GP_MIX_PIXELS_xRGB8888(pixel, pix, perc);
	GP_PutPixel_Raw_32BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_RGBA8888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_32BPP(context, x, y);
	pix = GP_MIX_PIXELS_RGBA8888(pixel, pix, perc);
	GP_PutPixel_Raw_32BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_RGB888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_24BPP(context, x, y);
	pix = GP_MIX_PIXELS_RGB888(pixel, pix, perc);
	GP_PutPixel_Raw_24BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_BGR888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_24BPP(context, x, y);
	pix = GP_MIX_PIXELS_BGR888(pixel, pix, perc);
	GP_PutPixel_Raw_24BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_RGB555(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_16BPP(context, x, y);
	pix = GP_MIX_PIXELS_RGB555(pixel, pix, perc);
	GP_PutPixel_Raw_16BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_RGB565(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_16BPP(context, x, y);
	pix = GP_MIX_PIXELS_RGB565(pixel, pix, perc);
	GP_PutPixel_Raw_16BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_RGB666(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_18BPP_LE(context, x, y);
	pix = GP_MIX_PIXELS_RGB666(pixel, pix, perc);
	GP_PutPixel_Raw_18BPP_LE(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_RGB332(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_8BPP(context, x, y);
	pix = GP_MIX_PIXELS_RGB332(pixel, pix, perc);
	GP_PutPixel_Raw_8BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_CMYK8888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_32BPP(context, x, y);
	pix = GP_MIX_PIXELS_CMYK8888(pixel, pix, perc);
	GP_PutPixel_Raw_32BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_P2(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_2BPP_LE(context, x, y);
	pix = GP_MIX_PIXELS_P2(pixel, pix, perc);
	GP_PutPixel_Raw_2BPP_LE(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_P4(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_4BPP_LE(context, x, y);
	pix = GP_MIX_PIXELS_P4(pixel, pix, perc);
	GP_PutPixel_Raw_4BPP_LE(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_P8(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_8BPP(context, x, y);
	pix = GP_MIX_PIXELS_P8(pixel, pix, perc);
	GP_PutPixel_Raw_8BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_G1(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_1BPP_LE(context, x, y);
	pix = GP_MIX_PIXELS_G1(pixel, pix, perc);
	GP_PutPixel_Raw_1BPP_LE(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_G2(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_2BPP_LE(context, x, y);
	pix = GP_MIX_PIXELS_G2(pixel, pix, perc);
	GP_PutPixel_Raw_2BPP_LE(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_G4(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_4BPP_LE(context, x, y);
	pix = GP_MIX_PIXELS_G4(pixel, pix, perc);
	GP_PutPixel_Raw_4BPP_LE(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_G8(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_8BPP(context, x, y);
	pix = GP_MIX_PIXELS_G8(pixel, pix, perc);
	GP_PutPixel_Raw_8BPP(context, x, y, pix);
}

static inline void GP_MixPixel_Raw_G16(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	GP_Pixel pix = GP_GetPixel_Raw_16BPP(context, x, y);
	pix = GP_MIX_PIXELS_G16(pixel, pix, perc);
	GP_PutPixel_Raw_16BPP(context, x, y, pix);
}


static inline void GP_MixPixel_Raw_Clipped_xRGB8888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_xRGB8888(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_RGBA8888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_RGBA8888(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_RGB888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_RGB888(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_BGR888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_BGR888(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_RGB555(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_RGB555(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_RGB565(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_RGB565(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_RGB666(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_RGB666(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_RGB332(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_RGB332(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_CMYK8888(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_CMYK8888(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_P2(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_P2(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_P4(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_P4(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_P8(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_P8(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_G1(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_G1(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_G2(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_G2(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_G4(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_G4(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_G8(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_G8(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw_Clipped_G16(GP_Context *context,
			GP_Coord x, GP_Coord y, GP_Pixel pixel, uint8_t perc)
{
	if (GP_PIXEL_IS_CLIPPED(context, x, y))
		return;

	GP_MixPixel_Raw_G16(context, x, y, pixel, perc);
}

static inline void GP_MixPixel_Raw(GP_Context *context, GP_Coord x, GP_Coord y,
                                   GP_Pixel pixel, uint8_t perc)
{
	switch (context->pixel_type) {
	case GP_PIXEL_xRGB8888:
				GP_MixPixel_Raw_xRGB8888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGBA8888:
				GP_MixPixel_Raw_RGBA8888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB888:
				GP_MixPixel_Raw_RGB888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_BGR888:
				GP_MixPixel_Raw_BGR888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB555:
				GP_MixPixel_Raw_RGB555(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB565:
				GP_MixPixel_Raw_RGB565(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB666:
				GP_MixPixel_Raw_RGB666(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB332:
				GP_MixPixel_Raw_RGB332(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_CMYK8888:
				GP_MixPixel_Raw_CMYK8888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_P2:
				GP_MixPixel_Raw_P2(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_P4:
				GP_MixPixel_Raw_P4(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_P8:
				GP_MixPixel_Raw_P8(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G1:
				GP_MixPixel_Raw_G1(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G2:
				GP_MixPixel_Raw_G2(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G4:
				GP_MixPixel_Raw_G4(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G8:
				GP_MixPixel_Raw_G8(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G16:
				GP_MixPixel_Raw_G16(context, x, y, pixel, perc);
	break;
	default:
		GP_ABORT("Unknown pixeltype");
	}
}

static inline void GP_MixPixel_Raw_Clipped(GP_Context *context,
                                           GP_Coord x, GP_Coord y,
                                           GP_Pixel pixel, uint8_t perc)
{
	switch (context->pixel_type) {
	case GP_PIXEL_xRGB8888:
		GP_MixPixel_Raw_Clipped_xRGB8888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGBA8888:
		GP_MixPixel_Raw_Clipped_RGBA8888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB888:
		GP_MixPixel_Raw_Clipped_RGB888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_BGR888:
		GP_MixPixel_Raw_Clipped_BGR888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB555:
		GP_MixPixel_Raw_Clipped_RGB555(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB565:
		GP_MixPixel_Raw_Clipped_RGB565(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB666:
		GP_MixPixel_Raw_Clipped_RGB666(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_RGB332:
		GP_MixPixel_Raw_Clipped_RGB332(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_CMYK8888:
		GP_MixPixel_Raw_Clipped_CMYK8888(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_P2:
		GP_MixPixel_Raw_Clipped_P2(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_P4:
		GP_MixPixel_Raw_Clipped_P4(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_P8:
		GP_MixPixel_Raw_Clipped_P8(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G1:
		GP_MixPixel_Raw_Clipped_G1(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G2:
		GP_MixPixel_Raw_Clipped_G2(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G4:
		GP_MixPixel_Raw_Clipped_G4(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G8:
		GP_MixPixel_Raw_Clipped_G8(context, x, y, pixel, perc);
	break;
	case GP_PIXEL_G16:
		GP_MixPixel_Raw_Clipped_G16(context, x, y, pixel, perc);
	break;
	default:
		GP_ABORT("Unknown pixeltype");
	}
}

#endif  /* GP_MIXPIXELS_GEN_H */
