/*****************************************************************************
 * This file is part of gfxprim library.                                     *
 *                                                                           *
 * Gfxprim is free software; you can redistribute it and/or                  *
 * modify it under the terms of the GNU Lesser General Public                *
 * License as published by the Free Software Foundation; either              *
 * version 2.1 of the License, or (at your option) any later version.        *
 *                                                                           *
 * Gfxprim is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with gfxprim; if not, write to the Free Software            *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,                        *
 * Boston, MA  02110-1301  USA                                               *
 *                                                                           *
 * Copyright (C) 2009-2013 Cyril Hrubis <metan@ucw.cz>                       *
 *                                                                           *
 *****************************************************************************/

/*
 * GP_MixPixels2.gen.h
 *
 * GENERATED on Mon Dec 23 05:05:54 2013 from GP_MixPixels2.gen.h.t
 * DO NOT MODIFY THIS FILE DIRECTLY!
 *
 * Macros to mix two pixels. The source must have alpha channel.
 */
#ifndef GP_MIXPIXELS2_GEN_H
#define GP_MIXPIXELS2_GEN_H
#include "core/GP_GammaCorrection.h"
#include "core/GP_Pixel.h"

//TODO: Fix blit where both source and destination have alpha channel

static inline GP_Pixel GP_MixPixels_RGBA8888_xRGB8888(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_xRGB8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_xRGB8888(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_RGBA8888(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_RGBA8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_RGBA8888(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_RGB888(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_RGB888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_RGB888(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_BGR888(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_BGR888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_BGR888(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_RGB555(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_RGB555_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_RGB555(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_RGB565(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_RGB565_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_RGB565(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_RGB666(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_RGB666_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_RGB666(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_RGB332(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_RGB332_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_RGB332(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_CMYK8888(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_CMYK8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_CMYK8888(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_G1(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_G1_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_G1(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_G2(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_G2_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_G2(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_G4(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_G4_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_G4(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_G8(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_G8_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_G8(dst_rgb, res);

	return res;
}

static inline GP_Pixel GP_MixPixels_RGBA8888_G16(GP_Pixel src, GP_Pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_Pixel_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	GP_Pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_Pixel_RGBA8888_TO_RGB888(src, src_rgb);
	GP_Pixel_G16_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_Pixel_GET_R_RGB888(src_rgb);
	sg = GP_Pixel_GET_G_RGB888(src_rgb);
	sb = GP_Pixel_GET_B_RGB888(src_rgb);

	dr = GP_Pixel_GET_R_RGB888(dst_rgb);
	dg = GP_Pixel_GET_G_RGB888(dst_rgb);
	db = GP_Pixel_GET_B_RGB888(dst_rgb);

	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_Pixel_CREATE_RGB888(dr, dg, db);

	GP_Pixel_RGB888_TO_G16(dst_rgb, res);

	return res;
}


#endif  /* GP_MIXPIXELS2_GEN_H */
