/*****************************************************************************
 * This file is part of gfxprim library.                                     *
 *                                                                           *
 * Gfxprim is free software; you can redistribute it and/or                  *
 * modify it under the terms of the GNU Lesser General Public                *
 * License as published by the Free Software Foundation; either              *
 * version 2.1 of the License, or (at your option) any later version.        *
 *                                                                           *
 * Gfxprim is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Lesser General Public License for more details.                           *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public          *
 * License along with gfxprim; if not, write to the Free Software            *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,                        *
 * Boston, MA  02110-1301  USA                                               *
 *                                                                           *
 * Copyright (C) 2011 Tomas Gavenciak <gavento@ucw.cz>                       *
 * Copyright (C) 2013 Cyril Hrubis <metan@ucw.cz>                            *
 *                                                                           *
 *****************************************************************************/

/*
 * GP_Pixel.gen.h
 *
 * GENERATED on Mon Dec 23 05:05:53 2013 from GP_Pixel.gen.h.t
 * DO NOT MODIFY THIS FILE DIRECTLY!
 *
 * Pixel type definitions and functions.
 * Do not include directly, use GP_Pixel.h
 */
#ifndef GP_PIXEL_GEN_H
#define GP_PIXEL_GEN_H
/*
 * List of all known pixel types
 */
typedef enum GP_PixelType {
	GP_PIXEL_UNKNOWN,
	GP_PIXEL_xRGB8888,
	GP_PIXEL_RGBA8888,
	GP_PIXEL_RGB888,
	GP_PIXEL_BGR888,
	GP_PIXEL_RGB555,
	GP_PIXEL_RGB565,
	GP_PIXEL_RGB666,
	GP_PIXEL_RGB332,
	GP_PIXEL_CMYK8888,
	GP_PIXEL_P2,
	GP_PIXEL_P4,
	GP_PIXEL_P8,
	GP_PIXEL_G1,
	GP_PIXEL_G2,
	GP_PIXEL_G4,
	GP_PIXEL_G8,
	GP_PIXEL_G16,
	GP_PIXEL_MAX,
} GP_PixelType;

#define GP_PIXEL_UNKNOWN GP_PIXEL_UNKNOWN
#define GP_PIXEL_xRGB8888 GP_PIXEL_xRGB8888
#define GP_PIXEL_RGBA8888 GP_PIXEL_RGBA8888
#define GP_PIXEL_RGB888 GP_PIXEL_RGB888
#define GP_PIXEL_BGR888 GP_PIXEL_BGR888
#define GP_PIXEL_RGB555 GP_PIXEL_RGB555
#define GP_PIXEL_RGB565 GP_PIXEL_RGB565
#define GP_PIXEL_RGB666 GP_PIXEL_RGB666
#define GP_PIXEL_RGB332 GP_PIXEL_RGB332
#define GP_PIXEL_CMYK8888 GP_PIXEL_CMYK8888
#define GP_PIXEL_P2 GP_PIXEL_P2
#define GP_PIXEL_P4 GP_PIXEL_P4
#define GP_PIXEL_P8 GP_PIXEL_P8
#define GP_PIXEL_G1 GP_PIXEL_G1
#define GP_PIXEL_G2 GP_PIXEL_G2
#define GP_PIXEL_G4 GP_PIXEL_G4
#define GP_PIXEL_G8 GP_PIXEL_G8
#define GP_PIXEL_G16 GP_PIXEL_G16

/* Automatically generated code for pixel type xRGB8888
 *
 * Size (bpp): 32 (32BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: BBBBBBBBGGGGGGGGRRRRRRRRxxxxxxxx
 * Channels:
 *   R  offset:16 size:8
 *   G  offset:8 size:8
 *   B  offset:0 size:8
 */

/*
 * macros to get channels of pixel type xRGB8888
 */
#define GP_Pixel_GET_R_xRGB8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_Pixel_GET_G_xRGB8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_Pixel_GET_B_xRGB8888(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel xRGB8888 in a context
 */
#define GP_PIXEL_ADDR_xRGB8888(context, x, y) GP_PIXEL_ADDR_32BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_xRGB8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros to create GP_Pixel of pixel type xRGB8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_xRGB8888(R, G, B) (0\
	+ ((R) << 16) \
	+ ((G) << 8) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGBA8888
 *
 * Size (bpp): 32 (32BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: AAAAAAAABBBBBBBBGGGGGGGGRRRRRRRR
 * Channels:
 *   R  offset:24 size:8
 *   G  offset:16 size:8
 *   B  offset:8 size:8
 *   A  offset:0 size:8
 */

/*
 * macros to get channels of pixel type RGBA8888
 */
#define GP_Pixel_GET_R_RGBA8888(p) (GP_GET_BITS(24, 8, (p)))
#define GP_Pixel_GET_G_RGBA8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_Pixel_GET_B_RGBA8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_Pixel_GET_A_RGBA8888(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel RGBA8888 in a context
 */
#define GP_PIXEL_ADDR_RGBA8888(context, x, y) GP_PIXEL_ADDR_32BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGBA8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros to create GP_Pixel of pixel type RGBA8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_RGBA8888(R, G, B, A) (0\
	+ ((R) << 24) \
	+ ((G) << 16) \
	+ ((B) << 8) \
	+ ((A) << 0) \
	)

/* Automatically generated code for pixel type RGB888
 *
 * Size (bpp): 24 (24BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: BBBBBBBBGGGGGGGGRRRRRRRR
 * Channels:
 *   R  offset:16 size:8
 *   G  offset:8 size:8
 *   B  offset:0 size:8
 */

/*
 * macros to get channels of pixel type RGB888
 */
#define GP_Pixel_GET_R_RGB888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_Pixel_GET_G_RGB888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_Pixel_GET_B_RGB888(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel RGB888 in a context
 */
#define GP_PIXEL_ADDR_RGB888(context, x, y) GP_PIXEL_ADDR_24BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB888(x) GP_PIXEL_ADDR_OFFSET_24BPP(x)

/*
 * macros to create GP_Pixel of pixel type RGB888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_RGB888(R, G, B) (0\
	+ ((R) << 16) \
	+ ((G) << 8) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type BGR888
 *
 * Size (bpp): 24 (24BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: RRRRRRRRGGGGGGGGBBBBBBBB
 * Channels:
 *   B  offset:16 size:8
 *   G  offset:8 size:8
 *   R  offset:0 size:8
 */

/*
 * macros to get channels of pixel type BGR888
 */
#define GP_Pixel_GET_B_BGR888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_Pixel_GET_G_BGR888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_Pixel_GET_R_BGR888(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel BGR888 in a context
 */
#define GP_PIXEL_ADDR_BGR888(context, x, y) GP_PIXEL_ADDR_24BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_BGR888(x) GP_PIXEL_ADDR_OFFSET_24BPP(x)

/*
 * macros to create GP_Pixel of pixel type BGR888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_BGR888(B, G, R) (0\
	+ ((B) << 16) \
	+ ((G) << 8) \
	+ ((R) << 0) \
	)

/* Automatically generated code for pixel type RGB555
 *
 * Size (bpp): 16 (16BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: BBBBBGGGGGRRRRRx
 * Channels:
 *   R  offset:10 size:5
 *   G  offset:5 size:5
 *   B  offset:0 size:5
 */

/*
 * macros to get channels of pixel type RGB555
 */
#define GP_Pixel_GET_R_RGB555(p) (GP_GET_BITS(10, 5, (p)))
#define GP_Pixel_GET_G_RGB555(p) (GP_GET_BITS(5, 5, (p)))
#define GP_Pixel_GET_B_RGB555(p) (GP_GET_BITS(0, 5, (p)))
/*
 * macros to get address and bit-offset of a pixel RGB555 in a context
 */
#define GP_PIXEL_ADDR_RGB555(context, x, y) GP_PIXEL_ADDR_16BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB555(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create GP_Pixel of pixel type RGB555 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_RGB555(R, G, B) (0\
	+ ((R) << 10) \
	+ ((G) << 5) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGB565
 *
 * Size (bpp): 16 (16BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: BBBBBGGGGGGRRRRR
 * Channels:
 *   R  offset:11 size:5
 *   G  offset:5 size:6
 *   B  offset:0 size:5
 */

/*
 * macros to get channels of pixel type RGB565
 */
#define GP_Pixel_GET_R_RGB565(p) (GP_GET_BITS(11, 5, (p)))
#define GP_Pixel_GET_G_RGB565(p) (GP_GET_BITS(5, 6, (p)))
#define GP_Pixel_GET_B_RGB565(p) (GP_GET_BITS(0, 5, (p)))
/*
 * macros to get address and bit-offset of a pixel RGB565 in a context
 */
#define GP_PIXEL_ADDR_RGB565(context, x, y) GP_PIXEL_ADDR_16BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB565(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create GP_Pixel of pixel type RGB565 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_RGB565(R, G, B) (0\
	+ ((R) << 11) \
	+ ((G) << 5) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGB666
 *
 * Size (bpp): 18 (18BPP_LE)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: BBBBBBGGGGGGRRRRRR
 * Channels:
 *   R  offset:12 size:6
 *   G  offset:6 size:6
 *   B  offset:0 size:6
 */

/*
 * macros to get channels of pixel type RGB666
 */
#define GP_Pixel_GET_R_RGB666(p) (GP_GET_BITS(12, 6, (p)))
#define GP_Pixel_GET_G_RGB666(p) (GP_GET_BITS(6, 6, (p)))
#define GP_Pixel_GET_B_RGB666(p) (GP_GET_BITS(0, 6, (p)))
/*
 * macros to get address and bit-offset of a pixel RGB666 in a context
 */
#define GP_PIXEL_ADDR_RGB666(context, x, y) GP_PIXEL_ADDR_18BPP_LE(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB666(x) GP_PIXEL_ADDR_OFFSET_18BPP_LE(x)

/*
 * macros to create GP_Pixel of pixel type RGB666 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_RGB666(R, G, B) (0\
	+ ((R) << 12) \
	+ ((G) << 6) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGB332
 *
 * Size (bpp): 8 (8BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: BBGGGRRR
 * Channels:
 *   R  offset:5 size:3
 *   G  offset:2 size:3
 *   B  offset:0 size:2
 */

/*
 * macros to get channels of pixel type RGB332
 */
#define GP_Pixel_GET_R_RGB332(p) (GP_GET_BITS(5, 3, (p)))
#define GP_Pixel_GET_G_RGB332(p) (GP_GET_BITS(2, 3, (p)))
#define GP_Pixel_GET_B_RGB332(p) (GP_GET_BITS(0, 2, (p)))
/*
 * macros to get address and bit-offset of a pixel RGB332 in a context
 */
#define GP_PIXEL_ADDR_RGB332(context, x, y) GP_PIXEL_ADDR_8BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB332(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros to create GP_Pixel of pixel type RGB332 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_RGB332(R, G, B) (0\
	+ ((R) << 5) \
	+ ((G) << 2) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type CMYK8888
 *
 * Size (bpp): 32 (32BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: CCCCCCCCMMMMMMMMYYYYYYYYKKKKKKKK
 * Channels:
 *   K  offset:24 size:8
 *   Y  offset:16 size:8
 *   M  offset:8 size:8
 *   C  offset:0 size:8
 */

/*
 * macros to get channels of pixel type CMYK8888
 */
#define GP_Pixel_GET_K_CMYK8888(p) (GP_GET_BITS(24, 8, (p)))
#define GP_Pixel_GET_Y_CMYK8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_Pixel_GET_M_CMYK8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_Pixel_GET_C_CMYK8888(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel CMYK8888 in a context
 */
#define GP_PIXEL_ADDR_CMYK8888(context, x, y) GP_PIXEL_ADDR_32BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_CMYK8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros to create GP_Pixel of pixel type CMYK8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_CMYK8888(K, Y, M, C) (0\
	+ ((K) << 24) \
	+ ((Y) << 16) \
	+ ((M) << 8) \
	+ ((C) << 0) \
	)

/* Automatically generated code for pixel type P2
 *
 * Size (bpp): 2 (2BPP_LE)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: PP
 * Channels:
 *   P  offset:0 size:2
 */

/*
 * macros to get channels of pixel type P2
 */
#define GP_Pixel_GET_P_P2(p) (GP_GET_BITS(0, 2, (p)))
/*
 * macros to get address and bit-offset of a pixel P2 in a context
 */
#define GP_PIXEL_ADDR_P2(context, x, y) GP_PIXEL_ADDR_2BPP_LE(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_P2(x) GP_PIXEL_ADDR_OFFSET_2BPP_LE(x)

/*
 * macros to create GP_Pixel of pixel type P2 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_P2(P) (0\
	+ ((P) << 0) \
	)

/* Automatically generated code for pixel type P4
 *
 * Size (bpp): 4 (4BPP_LE)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: PPPP
 * Channels:
 *   P  offset:0 size:4
 */

/*
 * macros to get channels of pixel type P4
 */
#define GP_Pixel_GET_P_P4(p) (GP_GET_BITS(0, 4, (p)))
/*
 * macros to get address and bit-offset of a pixel P4 in a context
 */
#define GP_PIXEL_ADDR_P4(context, x, y) GP_PIXEL_ADDR_4BPP_LE(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_P4(x) GP_PIXEL_ADDR_OFFSET_4BPP_LE(x)

/*
 * macros to create GP_Pixel of pixel type P4 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_P4(P) (0\
	+ ((P) << 0) \
	)

/* Automatically generated code for pixel type P8
 *
 * Size (bpp): 8 (8BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: PPPPPPPP
 * Channels:
 *   P  offset:0 size:8
 */

/*
 * macros to get channels of pixel type P8
 */
#define GP_Pixel_GET_P_P8(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel P8 in a context
 */
#define GP_PIXEL_ADDR_P8(context, x, y) GP_PIXEL_ADDR_8BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_P8(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros to create GP_Pixel of pixel type P8 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_P8(P) (0\
	+ ((P) << 0) \
	)

/* Automatically generated code for pixel type G1
 *
 * Size (bpp): 1 (1BPP_LE)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: V
 * Channels:
 *   V  offset:0 size:1
 */

/*
 * macros to get channels of pixel type G1
 */
#define GP_Pixel_GET_V_G1(p) (GP_GET_BITS(0, 1, (p)))
/*
 * macros to get address and bit-offset of a pixel G1 in a context
 */
#define GP_PIXEL_ADDR_G1(context, x, y) GP_PIXEL_ADDR_1BPP_LE(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_G1(x) GP_PIXEL_ADDR_OFFSET_1BPP_LE(x)

/*
 * macros to create GP_Pixel of pixel type G1 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_G1(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G2
 *
 * Size (bpp): 2 (2BPP_LE)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: VV
 * Channels:
 *   V  offset:0 size:2
 */

/*
 * macros to get channels of pixel type G2
 */
#define GP_Pixel_GET_V_G2(p) (GP_GET_BITS(0, 2, (p)))
/*
 * macros to get address and bit-offset of a pixel G2 in a context
 */
#define GP_PIXEL_ADDR_G2(context, x, y) GP_PIXEL_ADDR_2BPP_LE(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_G2(x) GP_PIXEL_ADDR_OFFSET_2BPP_LE(x)

/*
 * macros to create GP_Pixel of pixel type G2 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_G2(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G4
 *
 * Size (bpp): 4 (4BPP_LE)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: VVVV
 * Channels:
 *   V  offset:0 size:4
 */

/*
 * macros to get channels of pixel type G4
 */
#define GP_Pixel_GET_V_G4(p) (GP_GET_BITS(0, 4, (p)))
/*
 * macros to get address and bit-offset of a pixel G4 in a context
 */
#define GP_PIXEL_ADDR_G4(context, x, y) GP_PIXEL_ADDR_4BPP_LE(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_G4(x) GP_PIXEL_ADDR_OFFSET_4BPP_LE(x)

/*
 * macros to create GP_Pixel of pixel type G4 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_G4(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G8
 *
 * Size (bpp): 8 (8BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: VVVVVVVV
 * Channels:
 *   V  offset:0 size:8
 */

/*
 * macros to get channels of pixel type G8
 */
#define GP_Pixel_GET_V_G8(p) (GP_GET_BITS(0, 8, (p)))
/*
 * macros to get address and bit-offset of a pixel G8 in a context
 */
#define GP_PIXEL_ADDR_G8(context, x, y) GP_PIXEL_ADDR_8BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_G8(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros to create GP_Pixel of pixel type G8 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_G8(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G16
 *
 * Size (bpp): 16 (16BPP)
 * Bit endian: GP_BIT_ENDIAN_LE
 * Pixel structure: VVVVVVVVVVVVVVVV
 * Channels:
 *   V  offset:0 size:16
 */

/*
 * macros to get channels of pixel type G16
 */
#define GP_Pixel_GET_V_G16(p) (GP_GET_BITS(0, 16, (p)))
/*
 * macros to get address and bit-offset of a pixel G16 in a context
 */
#define GP_PIXEL_ADDR_G16(context, x, y) GP_PIXEL_ADDR_16BPP(context, x, y)
#define GP_PIXEL_ADDR_OFFSET_G16(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create GP_Pixel of pixel type G16 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_Pixel_CREATE_G16(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros for branching on PixelType (similar to GP_FnPerBpp macros)
 */

#define GP_FN_PER_PIXELTYPE(FN_NAME, type, ...)\
	switch (type) { \
		case GP_PIXEL_xRGB8888:\
			FN_NAME##_xRGB8888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGBA8888:\
			FN_NAME##_RGBA8888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB888:\
			FN_NAME##_RGB888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_BGR888:\
			FN_NAME##_BGR888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB555:\
			FN_NAME##_RGB555(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB565:\
			FN_NAME##_RGB565(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB666:\
			FN_NAME##_RGB666(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB332:\
			FN_NAME##_RGB332(__VA_ARGS__);\
			break;\
		case GP_PIXEL_CMYK8888:\
			FN_NAME##_CMYK8888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_P2:\
			FN_NAME##_P2(__VA_ARGS__);\
			break;\
		case GP_PIXEL_P4:\
			FN_NAME##_P4(__VA_ARGS__);\
			break;\
		case GP_PIXEL_P8:\
			FN_NAME##_P8(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G1:\
			FN_NAME##_G1(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G2:\
			FN_NAME##_G2(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G4:\
			FN_NAME##_G4(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G8:\
			FN_NAME##_G8(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G16:\
			FN_NAME##_G16(__VA_ARGS__);\
			break;\
		default: GP_ABORT("Invalid PixelType %d", type);\
	}

#define GP_FN_RET_PER_PIXELTYPE(FN_NAME, type, ...)\
	switch (type) { \
		case GP_PIXEL_xRGB8888:\
			return FN_NAME##_xRGB8888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGBA8888:\
			return FN_NAME##_RGBA8888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB888:\
			return FN_NAME##_RGB888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_BGR888:\
			return FN_NAME##_BGR888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB555:\
			return FN_NAME##_RGB555(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB565:\
			return FN_NAME##_RGB565(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB666:\
			return FN_NAME##_RGB666(__VA_ARGS__);\
			break;\
		case GP_PIXEL_RGB332:\
			return FN_NAME##_RGB332(__VA_ARGS__);\
			break;\
		case GP_PIXEL_CMYK8888:\
			return FN_NAME##_CMYK8888(__VA_ARGS__);\
			break;\
		case GP_PIXEL_P2:\
			return FN_NAME##_P2(__VA_ARGS__);\
			break;\
		case GP_PIXEL_P4:\
			return FN_NAME##_P4(__VA_ARGS__);\
			break;\
		case GP_PIXEL_P8:\
			return FN_NAME##_P8(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G1:\
			return FN_NAME##_G1(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G2:\
			return FN_NAME##_G2(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G4:\
			return FN_NAME##_G4(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G8:\
			return FN_NAME##_G8(__VA_ARGS__);\
			break;\
		case GP_PIXEL_G16:\
			return FN_NAME##_G16(__VA_ARGS__);\
			break;\
		default: GP_ABORT("Invalid PixelType %d", type);\
	}


#endif  /* GP_PIXEL_GEN_H */
